#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : scan.py
# Author             : Podalirius (@podalirius_)
# Date created       : 29 Jul 2022

import base64
import re
import requests


def is_target_a_windows_machine() -> bool:
    # if port 135 and 445 open
    pass


def is_target_a_windows_domain_controller() -> bool:
    # if port 135 and 445 and 88 and ldap/ldaps open
    pass


def is_http_accessible(target, port, config):
    url = "http://%s:%d/" % (target, port)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies
        )
        return True
    except Exception as e:
        return False


def is_tomcat_manager_accessible(target, port, config):
    path = "/manager/html"
    url = "http://%s:%d%s" % (target, port, path)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies
        )
        if r.status_code in [401]:
            return True
        else:
            return False
    except Exception as e:
        return False


def get_version_from_malformed_http_request(target, port, config):
    url = "http://%s:%d/{}" % (target, port)
    try:
        r = requests.get(
            url,
            timeout=config.request_timeout,
            proxies=config.request_proxies
        )
    except Exception as e:
        return None
    if r.status_code in [400, 404, 500]:
        # Bug triggered
        matched = re.search(b"(<h3>)Apache Tomcat(/)?([^<]+)(</h3>)", r.content)
        if matched is not None:
            _, _, version, _ = matched.groups()
            version = version.decode('utf-8')
            return version


def try_default_credentials(target, port, config):
    found_credentials = []
    url = "http://%s:%d/manager/html" % (target, port)
    try:
        couple_username_passwords = [
            {"username": "admin", "password": "admin"},
            {"username": "admin", "password": "tomcat"},
            {"username": "tomcat", "password": "tomcat"},
            {"username": "tomcat", "password": "s3cret"}
        ]
        for credentials in couple_username_passwords:
            auth_string = bytes(credentials["username"] + ':' + credentials["password"], 'utf-8')
            r = requests.post(
                url,
                headers={
                    "Authorization": "Basic " + base64.b64encode(auth_string).decode('utf-8')
                },
                timeout=config.request_timeout,
                proxies=config.request_proxies
            )
            if r.status_code in [200, 403]:
                found_credentials.append((r.status_code, credentials))
        return found_credentials
    except Exception as e:
        return found_credentials


def scan_worker(target, port, reporter, vulns_db, config):
    config.debug("scan_worker('%s', %d, ...)" % (target, port))
    result = {"target": target}
    if is_http_accessible(target, port, config):
        result["version"] = get_version_from_malformed_http_request(target, port, config)
        if result["version"] is not None:
            config.debug("Found version %s" % result["version"])

            result["manager_accessible"] = is_tomcat_manager_accessible(target, port, config)

            credentials = []
            if result["manager_accessible"]:
                config.debug("Manager is accessible")
                # Test for default credentials
                credentials = try_default_credentials(target, port, config)

            str_found_creds = []
            if len(credentials) != 0:
                for statuscode, creds in credentials:
                    str_found_creds.append("(username:\x1b[1;92m%s\x1b[0m password:\x1b[1;92m%s\x1b[0m)" % (creds["username"], creds["password"]))

            # List of cves
            cve_str = ""
            if config.list_cves_mode == True:
                cve_list = vulns_db.get_vulnerabilities_of_version_sorted_by_criticity(result["version"], colors=True, reverse=True)
                if len(cve_list) != 0:
                    cve_str = "CVEs: %s" % ', '.join(cve_list)

            print("[>] [Apache Tomcat/\x1b[1;95m%s\x1b[0m] on \x1b[1;93m%s\x1b[0m:\x1b[1;93m%d\x1b[0m (manager:%s) %s %s\x1b[0m " % (
                    result["version"],
                    target,
                    port,
                    ("\x1b[1;92maccessible\x1b[0m" if result["manager_accessible"] else "\x1b[1;91mnot accessible\x1b[0m"),
                    ' '.join(str_found_creds),
                    cve_str
                )
            )

            cve_list = vulns_db.get_vulnerabilities_of_version_sorted_by_criticity(result["version"], colors=False, reverse=True)
            credentials_str = "username:%s\npassword:%s" % (credentials[0][1]["username"], credentials[0][1]["password"])
            cve_str = ', '.join([cve["cve"]["id"] for cve in cve_list])

            reporter.report_result(
                target,
                port,
                result["version"],
                result["manager_accessible"],
                credentials_str,
                cve_str
            )





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : __main__.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022

import argparse
import os
import sys

from apachetomcatscanner.Reporter import Reporter
from apachetomcatscanner.Config import Config
from apachetomcatscanner.VulnerabilitiesDB import VulnerabilitiesDB
from apachetomcatscanner.utils.scan import scan_worker
from apachetomcatscanner.utils.targets import get_computers_from_domain
from sectools.network.ip import is_ipv4_cidr, is_ipv4_addr, is_ipv6_addr, expand_cidr, expand_port_range
from concurrent.futures import ThreadPoolExecutor


VERSION = "2.0"

banner = """Apache Tomcat Scanner v%s - by @podalirius_\n""" % VERSION


def load_targets(options, config):
    targets = []

    # Loading targets from domain computers
    if options.auth_domain is not None and options.auth_user is not None and (options.auth_password is not None or options.auth_hash is not None):
        if options.verbose:
            print("[debug] Loading targets from computers in the domain '%s'" % options.auth_domain)
        targets = get_computers_from_domain(
            auth_domain=options.auth_domain,
            auth_dc_ip=options.auth_dc_ip,
            auth_username=options.auth_user,
            auth_password=options.auth_password,
            auth_hashes=options.auth_hash
        )

    # Loading targets line by line from a targets file
    if options.targets_file is not None:
        if os.path.exists(options.targets_file):
            if options.verbose:
                print("[debug] Loading targets line by line from targets file '%s'" % options.targets_file)
            f = open(options.targets_file, "r")
            for line in f.readlines():
                targets.append(line.strip())
            f.close()
        else:
            print("[!] Could not open targets file '%s'" % options.targets_file)

    # Loading targets from --target option
    if len(options.target) != 0:
        if options.verbose:
            print("[debug] Loading targets from --target options")
        for target in options.target:
            targets.append(target)

    # Sort uniq on targets list
    targets = sorted(list(set(targets)))

    final_targets = []
    # Parsing target to filter IP/DNS/CIDR
    for target in targets:
        if is_ipv4_cidr(target):
            final_targets += expand_cidr(target)
        elif is_ipv4_addr(target):
            final_targets.append(target)
        elif is_ipv6_addr(target):
            final_targets.append(target)

    final_targets = sorted(list(set(final_targets)))
    return final_targets


def load_ports(options, config):
    ports = []
    if "," in options.target_ports:
        for port in options.target_ports.split(','):
            ports += expand_port_range(port.strip())
    else:
        ports = expand_port_range(options.target_ports.strip())
    ports = sorted(list(set(ports)))
    return ports


def parseArgs():
    print(banner)
    parser = argparse.ArgumentParser(description="A python script to scan for Apache Tomcat server vulnerabilities.")
    parser.add_argument("-v", "--verbose", default=False, action="store_true", help='Verbose mode. (default: False)')
    parser.add_argument("--debug", default=False, action="store_true", help='Debug mode, for huge verbosity. (default: False)')
    parser.add_argument("-C", "--list-cves", default=False, action="store_true", help='List CVE ids affecting each version found. (default: False)')
    parser.add_argument("-T", "--threads", default=8, type=int, help='Number of threads (default: 5)')

    parser.add_argument("--xlsx", default=None, type=str, help='Export results to XLSX')
    parser.add_argument("--json", default=None, type=str, help='Export results to JSON')

    group_configuration = parser.add_argument_group()
    group_configuration.add_argument("-PI", "--proxy-ip", default=None, type=str, help='Proxy IP.')
    group_configuration.add_argument("-PP", "--proxy-port", default=None, type=int, help='Proxy port')
    group_configuration.add_argument("-rt", "--request-timeout", default=1, type=int, help='')

    group_targets_source = parser.add_argument_group()
    group_targets_source.add_argument("-tf", "--targets-file", default=None, type=str, help='Path to file containing a line by line list of targets.')
    group_targets_source.add_argument("-tt", "--target", default=[], type=str, action='append', help='Target IP, FQDN or CIDR')
    group_targets_source.add_argument("-tp", "--target-ports", default="8080", type=str, help='Target ports to scan top search for Apache Tomcat servers.')
    group_targets_source.add_argument("-ad", "--auth-domain", default=None, type=str, help='Windows domain to authenticate to.')
    group_targets_source.add_argument("-ai", "--auth-dc-ip", default=None, type=str, help='IP of the domain controller.')
    group_targets_source.add_argument("-au", "--auth-user", default=None, type=str, help='Username of the domain account.')
    group_targets_source.add_argument("-ap", "--auth-password", default=None, type=str, help='Password of the domain account.')
    group_targets_source.add_argument("-ah", "--auth-hash", default=None, type=str, help='LM:NT hashes to pass the hash for this user.')

    args = parser.parse_args()

    if (args.targets_file is None) and (len(args.target) == 0) and (args.auth_domain is None and args.auth_user is None and (args.auth_password is None or args.auth_hash is None)):
        parser.print_help()
        print("\n[!] No targets specified.")
        sys.exit(0)

    if args.auth_password is not None and args.auth_hash is not None:
        parser.print_help()
        print("\n[!] Options --auth-password/--auth-hash are mutually exclusive.")
        sys.exit(0)

    return args


def main():
    options = parseArgs()

    config = Config()
    config.set_debug_mode(options.debug)
    config.set_request_timeout(options.request_timeout)
    config.set_request_proxies(options.proxy_ip, options.proxy_port)
    config.set_list_cves_mode(options.list_cves)

    reporter = Reporter(config=config)

    vulns_db = VulnerabilitiesDB(config=config)

    # Parsing targets and ports
    targets = load_targets(options, config)
    ports = load_ports(options, config)
    if options.proxy_ip is not None and options.proxy_port is not None:
        print("[+] Targeting %d ports on %d targets through proxy %s:%d" % (len(ports), len(targets), options.proxy_ip, options.proxy_port))
    else:
        print("[+] Targeting %d ports on %d targets" % (len(ports), len(targets)))

    # Exploring targets
    if len(targets) != 0 and options.threads != 0:
        print("[+] Searching for Apache Tomcats servers on specified targets ...")
        with ThreadPoolExecutor(max_workers=min(options.threads, len(targets))) as tp:
            for target in targets:
                for port in ports:
                    tp.submit(scan_worker, target, port, reporter, vulns_db, config)
        print("[+] All done!")

    if options.xlsx is not None:
        reporter.export_xlsx(options.xlsx)
    if options.json is not None:
        reporter.export_json(options.json)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Apache-Tomcat-Scanner.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022


from apachetomcatscanner.__main__ import main

if __name__ == '__main__':
    main()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : VulnerabilitiesDB.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022

import glob
import json
import os


class VulnerabilitiesDB(object):
    """
    Documentation for class VulnerabilitiesDB
    """

    def __init__(self, config):
        super(VulnerabilitiesDB, self).__init__()
        self.config = config
        self.cves = {}
        self.versions_to_cves = {}
        self.load()
    
    def load(self):
        self.cves = {}
        self.config.debug("Loading CVEs from JSON database ...")
        # Load all CVEs from JSON files
        for cve_json_file in glob.glob('%s/vulnerabilities/*/CVE-*.json' % os.path.dirname(__file__)):
            try:
                f = open(cve_json_file, 'r')
                cve = json.loads(f.read())
                f.close()

                if "cve" in cve.keys():
                    if "id" in cve["cve"].keys():
                        self.cves[cve["cve"]["id"]] = cve
            except Exception as e:
                pass
        self.config.debug("Loaded %d CVEs!" % len(self.cves.keys()))

        # Construct reverse lookup database from version to CVEs
        if len(self.cves.keys()) != 0:
            for cve_id, cve_data in self.cves.items():
                for version in cve_data["affected_versions"]:
                    if version["tag"] not in self.versions_to_cves.keys():
                        self.versions_to_cves[version["tag"]] = []
                    self.versions_to_cves[version["tag"]].append(cve_data)

    def get_vulnerabilities_of_version_sorted_by_criticity(self, version_tag, colors=False, reverse=False):
        colored_criticity = {
            "None": "\x1b[1;48;2;83;170;51;97m%s\x1b[0m",
            "Low": "\x1b[1;48;2;255;203;13;97m%s\x1b[0m",
            "Medium": "\x1b[1;48;2;249;160;9;97m%s\x1b[0m",
            "High": "\x1b[1;48;2;233;61;3;97m%s\x1b[0m",
            "Critical": "\x1b[1;48;2;45;45;45;97m%s\x1b[0m"
        }
        vulnerabilities = []
        if version_tag in self.versions_to_cves.keys():
            vulnerabilities = self.versions_to_cves[version_tag]
            vulnerabilities = sorted(vulnerabilities, key=lambda cve: cve["cvss"]["score"], reverse=reverse)
            if colors:
                vulnerabilities = [
                    colored_criticity[vuln["cvss"]["criticity"]] % vuln["cve"]["id"]
                    for vuln in vulnerabilities
                ]
        return vulnerabilities

    def get_vulnerabilities_of_version_sorted_by_year(self, version_tag, reverse=False):
        vulnerabilities = []
        if version_tag in self.versions_to_cves.keys():
            vulnerabilities = self.versions_to_cves[version_tag]
            vulnerabilities = sorted(vulnerabilities, key=lambda cve: cve["cve"]["year"], reverse=reverse)
        return vulnerabilities



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Target.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022

class Target(object):
    """
    Documentation for class Target
    """

    def __init__(self, target_host, target_port=8080):
        super(Target, self).__init__()
        self.target_host = target_host
        self.target_port = target_port




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : targets.py
# Author             : Podalirius (@podalirius_)
# Date created       : 29 Jul 2022

from impacket.smbconnection import SMBConnection
from impacket.spnego import SPNEGO_NegTokenInit, TypesMech
import binascii
import ldap3
import logging
import os
import ssl


def get_machine_name(auth_domain, auth_dc_ip):
    if auth_dc_ip is not None:
        s = SMBConnection(auth_dc_ip, auth_dc_ip)
    else:
        s = SMBConnection(auth_domain, auth_domain)
    try:
        s.login('', '')
    except Exception:
        if s.getServerName() == '':
            raise Exception('Error while anonymous logging into %s' % auth_domain)
    else:
        s.logoff()
    return s.getServerName()


def init_ldap_connection(target, tls_version, dc_ip, domain, username, password, lmhash, nthash, use_kerberos=False, use_ldaps=False, auth_key=None):
    user = '%s\\%s' % (domain, username)
    if tls_version is not None:
        use_ssl = True
        port = 636
        tls = ldap3.Tls(validate=ssl.CERT_NONE, version=tls_version)
    else:
        use_ssl = False
        port = 389
        tls = None
    ldap_server = ldap3.Server(target, get_info=ldap3.ALL, port=port, use_ssl=use_ssl, tls=tls)

    if use_kerberos:
        ldap_session = ldap3.Connection(ldap_server)
        ldap_session.bind()
        ldap3_kerberos_login(ldap_session, target, username, password, domain, lmhash, nthash, auth_key, kdcHost=dc_ip)
    elif len(nthash) != 0:
        if len(lmhash) == 0:
            lmhash = "aad3b435b51404eeaad3b435b51404ee"
        ldap_session = ldap3.Connection(ldap_server, user=user, password=lmhash + ":" + nthash, authentication=ldap3.NTLM, auto_bind=True)
    else:
        ldap_session = ldap3.Connection(ldap_server, user=user, password=password, authentication=ldap3.NTLM, auto_bind=True)

    return ldap_server, ldap_session


def init_ldap_session(auth_domain, auth_dc_ip, auth_username, auth_password, auth_lm_hash, auth_nt_hash, use_kerberos=False, use_ldaps=False):
    if use_kerberos:
        target_dc = get_machine_name(auth_domain, auth_dc_ip)
    else:
        target_dc = (auth_dc_ip if auth_dc_ip is not None else auth_domain)

    if use_ldaps is True:
        try:
            return init_ldap_connection(
                target=target_dc,
                tls_version=ssl.PROTOCOL_TLSv1_2,
                dc_ip=auth_dc_ip,
                domain=auth_domain,
                username=auth_username,
                password=auth_password,
                lmhash=auth_lm_hash,
                nthash=auth_nt_hash
            )
        except ldap3.core.exceptions.LDAPSocketOpenError:
            return init_ldap_connection(
                target=target_dc,
                tls_version=ssl.PROTOCOL_TLSv1,
                dc_ip=auth_dc_ip,
                domain=auth_domain,
                username=auth_username,
                password=auth_password,
                lmhash=auth_lm_hash,
                nthash=auth_nt_hash
            )
    else:
        return init_ldap_connection(
            target=target_dc,
            tls_version=None,
            dc_ip=auth_dc_ip,
            domain=auth_domain,
            username=auth_username,
            password=auth_password,
            lmhash=auth_lm_hash,
            nthash=auth_nt_hash
        )


def ldap3_kerberos_login(connection, target, user, password, domain='', lmhash='', nthash='', aesKey='', kdcHost=None, TGT=None, TGS=None, useCache=True):
    from pyasn1.codec.ber import encoder, decoder
    from pyasn1.type.univ import noValue
    """
    logins into the target system explicitly using Kerberos. Hashes are used if RC4_HMAC is supported.
    :param string user: username
    :param string password: password for the user
    :param string domain: domain where the account is valid for (required)
    :param string lmhash: LMHASH used to authenticate using hashes (password is not used)
    :param string nthash: NTHASH used to authenticate using hashes (password is not used)
    :param string aesKey: aes256-cts-hmac-sha1-96 or aes128-cts-hmac-sha1-96 used for Kerberos authentication
    :param string kdcHost: hostname or IP Address for the KDC. If None, the domain will be used (it needs to resolve tho)
    :param struct TGT: If there's a TGT available, send the structure here and it will be used
    :param struct TGS: same for TGS. See smb3.py for the format
    :param bool useCache: whether or not we should use the ccache for credentials lookup. If TGT or TGS are specified this is False
    :return: True, raises an Exception if error.
    """

    if lmhash != '' or nthash != '':
        if len(lmhash) % 2:
            lmhash = '0' + lmhash
        if len(nthash) % 2:
            nthash = '0' + nthash
        try:  # just in case they were converted already
            lmhash = binascii.unhexlify(lmhash)
            nthash = binascii.unhexlify(nthash)
        except TypeError:
            pass

    # Importing down here so pyasn1 is not required if kerberos is not used.
    from impacket.krb5.ccache import CCache
    from impacket.krb5.asn1 import AP_REQ, Authenticator, TGS_REP, seq_set
    from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS
    from impacket.krb5 import constants
    from impacket.krb5.types import Principal, KerberosTime, Ticket
    import datetime

    if TGT is not None or TGS is not None:
        useCache = False

    if useCache:
        try:
            ccache = CCache.loadFile(os.getenv('KRB5CCNAME'))
        except Exception as e:
            # No cache present
            print(e)
            pass
        else:
            # retrieve domain information from CCache file if needed
            if domain == '':
                domain = ccache.principal.realm['data'].decode('utf-8')
                logging.debug('Domain retrieved from CCache: %s' % domain)

            logging.debug('Using Kerberos Cache: %s' % os.getenv('KRB5CCNAME'))
            principal = 'ldap/%s@%s' % (target.upper(), domain.upper())

            creds = ccache.getCredential(principal)
            if creds is None:
                # Let's try for the TGT and go from there
                principal = 'krbtgt/%s@%s' % (domain.upper(), domain.upper())
                creds = ccache.getCredential(principal)
                if creds is not None:
                    TGT = creds.toTGT()
                    logging.debug('Using TGT from cache')
                else:
                    logging.debug('No valid credentials found in cache')
            else:
                TGS = creds.toTGS(principal)
                logging.debug('Using TGS from cache')

            # retrieve user information from CCache file if needed
            if user == '' and creds is not None:
                user = creds['client'].prettyPrint().split(b'@')[0].decode('utf-8')
                logging.debug('Username retrieved from CCache: %s' % user)
            elif user == '' and len(ccache.principal.components) > 0:
                user = ccache.principal.components[0]['data'].decode('utf-8')
                logging.debug('Username retrieved from CCache: %s' % user)

    # First of all, we need to get a TGT for the user
    userName = Principal(user, type=constants.PrincipalNameType.NT_PRINCIPAL.value)
    if TGT is None:
        if TGS is None:
            tgt, cipher, oldSessionKey, sessionKey = getKerberosTGT(userName, password, domain, lmhash, nthash, aesKey, kdcHost)
    else:
        tgt = TGT['KDC_REP']
        cipher = TGT['cipher']
        sessionKey = TGT['sessionKey']

    if TGS is None:
        serverName = Principal('ldap/%s' % target, type=constants.PrincipalNameType.NT_SRV_INST.value)
        tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(serverName, domain, kdcHost, tgt, cipher, sessionKey)
    else:
        tgs = TGS['KDC_REP']
        cipher = TGS['cipher']
        sessionKey = TGS['sessionKey']

        # Let's build a NegTokenInit with a Kerberos REQ_AP

    blob = SPNEGO_NegTokenInit()

    # Kerberos
    blob['MechTypes'] = [TypesMech['MS KRB5 - Microsoft Kerberos 5']]

    # Let's extract the ticket from the TGS
    tgs = decoder.decode(tgs, asn1Spec=TGS_REP())[0]
    ticket = Ticket()
    ticket.from_asn1(tgs['ticket'])

    # Now let's build the AP_REQ
    apReq = AP_REQ()
    apReq['pvno'] = 5
    apReq['msg-type'] = int(constants.ApplicationTagNumbers.AP_REQ.value)

    opts = []
    apReq['ap-options'] = constants.encodeFlags(opts)
    seq_set(apReq, 'ticket', ticket.to_asn1)

    authenticator = Authenticator()
    authenticator['authenticator-vno'] = 5
    authenticator['crealm'] = domain
    seq_set(authenticator, 'cname', userName.components_to_asn1)
    now = datetime.datetime.utcnow()

    authenticator['cusec'] = now.microsecond
    authenticator['ctime'] = KerberosTime.to_asn1(now)

    encodedAuthenticator = encoder.encode(authenticator)

    # Key Usage 11
    # AP-REQ Authenticator (includes application authenticator
    # subkey), encrypted with the application session key
    # (Section 5.5.1)
    encryptedEncodedAuthenticator = cipher.encrypt(sessionKey, 11, encodedAuthenticator, None)

    apReq['authenticator'] = noValue
    apReq['authenticator']['etype'] = cipher.enctype
    apReq['authenticator']['cipher'] = encryptedEncodedAuthenticator

    blob['MechToken'] = encoder.encode(apReq)

    request = ldap3.operation.bind.bind_operation(connection.version, ldap3.SASL, user, None, 'GSS-SPNEGO',
                                                  blob.getData())

    # Done with the Kerberos saga, now let's get into LDAP
    if connection.closed:  # try to open connection if closed
        connection.open(read_server_info=False)

    connection.sasl_in_progress = True
    response = connection.post_send_single_response(connection.send('bindRequest', request, None))
    connection.sasl_in_progress = False
    if response[0]['result'] != 0:
        raise Exception(response)

    connection.bound = True

    return True


def get_computers_from_domain(auth_domain, auth_dc_ip, auth_username, auth_password, auth_hashes):
    auth_lm_hash = ""
    auth_nt_hash = ""
    if auth_hashes is not None:
        if ":" in auth_hashes:
            auth_lm_hash = auth_hashes.split(":")[0]
            auth_nt_hash = auth_hashes.split(":")[1]
        else:
            auth_nt_hash = auth_hashes

    ldap_server, ldap_session = init_ldap_session(
        auth_domain=auth_domain,
        auth_dc_ip=auth_dc_ip,
        auth_username=auth_username,
        auth_password=auth_password,
        auth_lm_hash=auth_lm_hash,
        auth_nt_hash=auth_nt_hash,
        use_kerberos=False,
        use_ldaps=False
    )

    print("[>] Extracting all computers ...")

    computers = []
    target_dn = ldap_server.info.other["defaultNamingContext"]
    ldap_session.search(target_dn, "(objectCategory=computer)", attributes=["dNSHostName"])
    for entry in ldap_session.response:
        if entry['type'] != 'searchResEntry':
            continue
        computers.append(entry["attributes"]['dNSHostName'])

    print("[+] Found %d computers in the domain." % len(computers))

    return computers



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : test_environement.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022

import argparse
import os


instances = {
    "tomcat_8_5_81": "docker run --rm -d --name tomcat_8_5_81 -p 10001:8080 -v $(pwd)/versions/8.5.81/tomcat-users.xml:/usr/local/tomcat/conf/tomcat-users.xml tomcat:8.5.81-jdk8-corretto",
    "tomcat_7_0_59": "docker run --rm -d --name tomcat_7_0_59 -p 10002:8080 -v $(pwd)/versions/7.0.59/tomcat-users.xml:/usr/local/tomcat/conf/tomcat-users.xml tomcat:7.0.59",
    "tomcat_6_0_43": "docker run --rm -d --name tomcat_6_0_43 -p 10003:8080 -v $(pwd)/versions/6.0.43/tomcat-users.xml:/usr/local/tomcat/conf/tomcat-users.xml tomcat:6.0.43"
}


def parseArgs():
    parser = argparse.ArgumentParser(description="Description message")
    parser.add_argument("action", help='arg1 help message')
    parser.add_argument("-v", "--verbose", default=False, action="store_true", help='Verbose mode. (default: False)')
    options = parser.parse_args()
    if options.action.lower() not in ["start", "stop"]:
        pass
    return options


if __name__ == '__main__':
    options = parseArgs()

    if options.action.lower() == "start":
        for key in instances.keys():
            cmd = instances[key]
            if options.verbose:
                print(cmd)
            print("[+] Starting %s" % key)
            os.system(cmd)
    elif options.action.lower() == "stop":
        cmd = "docker stop %s" % ' '.join([key for key in instances.keys()])
        if options.verbose:
            print(cmd)
        os.system(cmd)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Reporter.py
# Author             : Podalirius (@podalirius_)
# Date created       : 31 Jul 2022

import json
import os.path
import xlsxwriter


class Reporter(object):
    """
    Documentation for class Reporter
    """

    data = {}

    def __init__(self, config):
        super(Reporter, self).__init__()
        self.config = config

    def report_result(self, computer_ip, computer_port, tomcat_version, manager_accessible, default_credentials, cves):
        computer_port = str(computer_port)
        if computer_ip not in self.data.keys():
            self.data[computer_ip] = {}
        if str(computer_port) not in self.data[computer_ip].keys():
            self.data[computer_ip][computer_port] = {}
        self.data[computer_ip][computer_port]["computer_ip"] = computer_ip
        self.data[computer_ip][computer_port]["computer_port"] = computer_port
        self.data[computer_ip][computer_port]["tomcat_version"] = tomcat_version
        self.data[computer_ip][computer_port]["manager_accessible"] = manager_accessible
        self.data[computer_ip][computer_port]["default_credentials"] = default_credentials
        self.data[computer_ip][computer_port]["cves"] = cves

    def export_xlsx(self, path_to_file):
        outdir = os.path.dirname(path_to_file)
        if len(outdir) != 0:
            if not os.path.exists(outdir):
                os.makedirs(os.path.dirname(path_to_file), exist_ok=True)

        workbook = xlsxwriter.Workbook(path_to_file)
        worksheet = workbook.add_worksheet()

        header_format = workbook.add_format({'bold': 1})
        header_fields = ["Computer IP", "Port", "Apache tomcat version", "Manager accessible", "Default credentials found", "CVEs on this version"]
        for k in range(len(header_fields)):
            worksheet.set_column(k, k + 1, len(header_fields[k]) + 3)
        worksheet.set_row(0, 20, header_format)
        worksheet.write_row(0, 0, header_fields)

        row_id = 1
        for computername in self.data.keys():
            computer = self.data[computername]
            for port in computer.keys():
                data = [
                    computer[port]["computer_ip"],
                    computer[port]["computer_port"],
                    computer[port]["tomcat_version"],
                    str(computer[port]["manager_accessible"]).upper(),
                    computer[port]["default_credentials"],
                    computer[port]["cves"]
                ]
                worksheet.write_row(row_id, 0, data)
                row_id += 1
        worksheet.autofilter(0, 0, row_id, len(header_fields) - 1)
        workbook.close()

    def export_json(self, path_to_file):
        outdir = os.path.dirname(path_to_file)
        if len(outdir) != 0:
            if not os.path.exists(outdir):
                os.makedirs(os.path.dirname(path_to_file), exist_ok=True)
        f = open(path_to_file, 'w')
        f.write(json.dumps(self.data, indent=4))
        f.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : setup.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022

import setuptools

with open('requirements.txt', 'r', encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt', 'r', encoding='utf-8') as f:
    requirements = [x.strip() for x in f.readlines()]

setuptools.setup(
    name="apachetomcatscanner",
    version="2.0",
    description="",
    url="https://github.com/p0dalirius/ApacheTomcatScanner",
    author="Podalirius",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author_email="podalirius@protonmail.com",
    packages=["apachetomcatscanner"],
    package_data={'apachetomcatscanner': ['apachetomcatscanner/']},
    include_package_data=True,
    license="GPL2",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=requirements,
    entry_points={
        'console_scripts': ['apachetomcatscanner=apachetomcatscanner.__main__:main']
    }
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Config.py
# Author             : Podalirius (@podalirius_)
# Date created       : 31 Jul 2022


class Config(object):
    """
    Documentation for class Config
    """

    request_timeout = 1
    request_proxies = {}

    list_cves_mode = False

    debug_mode = False
    verbose_mode = False

    def __init__(self):
        super(Config, self).__init__()

    def debug(self, msg):
        if self.debug_mode:
            print("[debug]", msg)

    # Get / Set functions

    def get_request_timeout(self):
        return self.request_timeout

    def set_request_timeout(self, value):
        self.request_timeout = value

    def get_list_cves_mode(self):
        return self.list_cves_mode

    def set_list_cves_mode(self, value):
        self.list_cves_mode = value

    def get_debug_mode(self):
        return self.debug_mode

    def set_debug_mode(self, value):
        if value == True:
            self.verbose_mode = True
        self.debug_mode = value

    def get_verbose_mode(self):
        return self.verbose_mode

    def set_verbose_mode(self, value):
        self.verbose_mode = value

    def get_request_proxies(self):
        return self.request_proxies

    def set_request_proxies(self, proxy_ip, proxy_port, protocol=None):
        if proxy_ip is not None and proxy_port is not None:
            if protocol is None:
                self.request_proxies = {
                    "http": "http://%s:%d/" % (proxy_ip, proxy_port),
                    "https": "https://%s:%d/" % (proxy_ip, proxy_port)
                }
            else:
                self.request_proxies[protocol] = "%s://%s:%d/" % (protocol, proxy_ip, proxy_port)
        return self.request_proxies

    def clear_request_proxies(self):
        self.request_proxies = {}
